@@ -14,6 +14,7 @@ from django.db.models.fields.related import (
     lazy_related_operation,
 )
 from django.db.models.query_utils import PathInfo
+from django.utils.functional import cached_property
 from django.utils.text import capfirst
 from django.utils.translation import gettext_lazy as _
 
@@ -590,16 +591,31 @@ class TaggableManager(RelatedField):
         pathinfos = []
         linkfield1 = self.through._meta.get_field("content_object")
         linkfield2 = self.through._meta.get_field(self.m2m_reverse_field_name())
-        if direct:
-            join1infos = linkfield1.get_reverse_path_info(
-                filtered_relation=filtered_relation
-            )
-            join2infos = linkfield2.get_path_info(filtered_relation=filtered_relation)
+        if django.VERSION >= (4, 1) and not filtered_relation:
+            # Django >= 4.1 provides cached path_infos and reverse_path_infos properties
+            # to use in preference to get_path_info / get_reverse_path_info when not
+            # passing a filtered_relation
+            if direct:
+                join1infos = linkfield1.reverse_path_infos
+                join2infos = linkfield2.path_infos
+            else:
+                join1infos = linkfield2.reverse_path_infos
+                join2infos = linkfield1.path_infos
         else:
-            join1infos = linkfield2.get_reverse_path_info(
-                filtered_relation=filtered_relation
-            )
-            join2infos = linkfield1.get_path_info(filtered_relation=filtered_relation)
+            if direct:
+                join1infos = linkfield1.get_reverse_path_info(
+                    filtered_relation=filtered_relation
+                )
+                join2infos = linkfield2.get_path_info(
+                    filtered_relation=filtered_relation
+                )
+            else:
+                join1infos = linkfield2.get_reverse_path_info(
+                    filtered_relation=filtered_relation
+                )
+                join2infos = linkfield1.get_path_info(
+                    filtered_relation=filtered_relation
+                )
         pathinfos.extend(join1infos)
         pathinfos.extend(join2infos)
         return pathinfos
@@ -621,11 +637,19 @@ class TaggableManager(RelatedField):
                     filtered_relation,
                 )
             ]
-            join2infos = linkfield.get_path_info(filtered_relation=filtered_relation)
+            if django.VERSION >= (4, 1) and not filtered_relation:
+                join2infos = linkfield.path_infos
+            else:
+                join2infos = linkfield.get_path_info(
+                    filtered_relation=filtered_relation
+                )
         else:
-            join1infos = linkfield.get_reverse_path_info(
-                filtered_relation=filtered_relation
-            )
+            if django.VERSION >= (4, 1) and not filtered_relation:
+                join1infos = linkfield.reverse_path_infos
+            else:
+                join1infos = linkfield.get_reverse_path_info(
+                    filtered_relation=filtered_relation
+                )
             join2infos = [
                 PathInfo(
                     opts,
@@ -651,6 +675,10 @@ class TaggableManager(RelatedField):
                 direct=True, filtered_relation=filtered_relation
             )
 
+    @cached_property
+    def path_infos(self):
+        return self.get_path_info()
+
     def get_reverse_path_info(self, filtered_relation=None):
         if self.use_gfk:
             return self._get_gfk_case_path_info(
@@ -661,6 +689,10 @@ class TaggableManager(RelatedField):
                 direct=False, filtered_relation=filtered_relation
             )
 
+    @cached_property
+    def reverse_path_infos(self):
+        return self.get_reverse_path_info()
+
     def get_joining_columns(self, reverse_join=False):
         if reverse_join:
             return ((self.model._meta.pk.column, "object_id"),)
